# -*- coding: utf-8 -*-

import json

from . import constants


class JSONRequest(object):
    raw_data = None
    json_data = None

    def __init__(self, server, stream, address, terminator=None):
        self.server = server
        self.stream = stream
        self.address = address
        self.terminator = terminator or '\n'

        self.stream.set_close_callback(self._on_close)
        # 开始等待数据
        self._read_message()

        # create_request_func_list
        for func in self.server.create_request_func_list:
            try:
                func(self)
            except Exception, e:
                self.server.logger.error('create_request_func_list raise exception. func: %s, e: %s, traceback: %s',
                                         func.__name__, e, __import__('traceback').format_exc())

    def write(self, data, callback=None):
        """
        发送数据(实现是放到发送队列)
        """
        self.stream.write(data, callback)

    def close(self, exc_info=False):
        """
        关闭连接
        """
        self.stream.close(exc_info)

    def make_rsp(self, *args, **kwargs):
        """
        生成rsp
        """
        return json.dumps(dict(*args, **kwargs)) + self.terminator

    def _on_close(self):
        # 链接被关闭的回调
        self.server.logger.debug('socket closed')

        # teardown_request_func_list
        for func in self.server.teardown_request_func_list:
            try:
                func(self)
            except Exception, e:
                self.server.logger.error('teardown_request_func_list raise exception. func: %s, e: %s, traceback: %s',
                                         func.__name__, e, __import__('traceback').format_exc())

    def _read_message(self):
        self.stream.read_until(self.terminator, self._on_read_over)

    def _on_read_over(self, data):
        """
        数据获取结束
        """
        self.raw_data = data
        self._handle_input_data()
        # 处理完了还是要继续保持可读
        self._read_message()

    def _handle_input_data(self):
        """
        因为udp和tcp的处理实在一样，所以还是抽离出来吧
        """
        self.server.logger.debug('raw_data: %s', self.raw_data)

        try:
            self.json_data = json.loads(self.raw_data)
        except Exception, e:
            error = 'parse raw_data fail. e: %s, raw_data: %s' % (e, self.raw_data)
            self.server.logger.error(error)
            self.write(self.make_rsp(ret=constants.RET_SYSTEM, error=error))
            self.close()
            return None

        if not isinstance(self.json_data, dict):
            error = 'json_data is not dict. json_data: %s' % self.json_data
            self.server.logger.error(error)
            self.write(self.make_rsp(ret=constants.RET_SYSTEM, error=error))
            self.close()
            return None

        if not self.json_data.get('endpoint'):
            error = 'endpoint is not in json_data. json_data: %s' % self.json_data
            self.server.logger.error(error)
            self.write(self.make_rsp(ret=constants.RET_SYSTEM, error=error))
            self.close()
            return None

        endpoint = self.json_data['endpoint']
        view_func = self.server.get_route_view_func(endpoint)
        if not view_func:
            error = 'endpoint is not valid. endpoint: %s' % endpoint
            self.server.logger.error(error)
            self.write(self.make_rsp(ret=constants.RET_SYSTEM, error=error))
            self.close()
            return None

        # before_request_func_list
        for func in self.server.before_request_func_list:
            try:
                func(self)
            except Exception, e:
                self.server.logger.error('before_request_func_list raise exception. func: %s, e: %s, traceback: %s',
                                         func.__name__, e, __import__('traceback').format_exc())
        view_func_exc = None
        view_func_result = None

        try:
            view_func_result = view_func(self)
        except Exception, e:
            error = 'view_func raise exception. endpoint: %s, view_func: %s, e: %s, traceback: %s' % (
                endpoint, view_func.__name__, e, __import__('traceback').format_exc())
            self.server.logger.error(error)
            view_func_exc = e
            self.write(self.make_rsp(ret=constants.RET_SYSTEM,
                                     error=error if self.server.debug else constants.ERROR_INTERNAL))

        # after_request_func_list
        for func in self.server.after_request_func_list:
            try:
                # 异常优先，以为一旦有异常，肯定没返回
                func(self, view_func_exc or view_func_result)
            except Exception, e:
                self.server.logger.error('after_request_func_list raise exception. func: %s, view_func_result: %s, e: %s, traceback: %s',
                                         func.__name__, view_func_result, e, __import__('traceback').format_exc())

        # 如果view_func异常，就要关闭socket
        if view_func_exc:
            self.close()

        return view_func_result
